/* $Workfile: OtxT1E1D.h $   $Revision: 79 $    $Modtime: 5/16/07 4:39p $  */
#ifndef OTX_T1E1_DEF_H
#define OTX_T1E1_DEF_H
/*******************************************************************************

  @doc PublicHwApi OtxT1E1Def

  Copyright (C) 1998-2003 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic Hardware Driver API

  @module
  OtxT1E1D | Constants, Macros, and Types for the T1/E1 Line Interface Device Functions

  @contents2
  OtxT1E1D.h |

  @index enum struct type | OtxT1E1Def

  @contents2
  OTX T1/E1 Line Interface Constants, Macros, and Types |

  @head3 OtxT1E1XXX, OtxT1XXX, and OtxE1XXX Macro, Constant, and Type Reference |

  @index enum struct type | OtxT1E1D

*******************************************************************************/

#include "OtxType.h"

#ifdef __cplusplus
  extern "C" {
#endif

#if defined(__linux__)
#pragma pack(4)
#elif defined(__BORLANDC__)
#pragma option push -a4
#else
#pragma pack(push,4)
#endif


/*******************************************************************************
  @enum 
    T1E1 Attributes
*******************************************************************************/
enum OtxT1E1AttributesE {
  OTX_ATTR_T1E1_TX_PULSE_MASK_SUB_PULSE_0    = OTX_ATTR_END + 0,  // @emem (OTX_ATTRIB_UINT32) Read/Write. First (of four) sub pulse shape value of the transmit pulse mask. This 5-bit value defines the level of the analog voltage in steps of about 80mV (for this sub pulse) on transmit pins of the E1T1 span.
  OTX_ATTR_T1E1_TX_PULSE_MASK_SUB_PULSE_1    = OTX_ATTR_END + 1,  // @emem (OTX_ATTRIB_UINT32) Read/Write. Second (of four) sub pulse shape value of the transmit pulse mask. This 5-bit value defines the level of the analog voltage in steps of about 80mV (for this sub pulse) on transmit pins of the E1T1 span.
  OTX_ATTR_T1E1_TX_PULSE_MASK_SUB_PULSE_2    = OTX_ATTR_END + 2,  // @emem (OTX_ATTRIB_UINT32) Read/Write. Third (of four) sub pulse shape value of the transmit pulse mask. This 5-bit value defines the level of the analog voltage in steps of about 80mV (for this sub pulse) on transmit pins of the E1T1 span.
  OTX_ATTR_T1E1_TX_PULSE_MASK_SUB_PULSE_3    = OTX_ATTR_END + 3,  // @emem (OTX_ATTRIB_UINT32) Read/Write. Forth (of four) sub pulse shape value of the transmit pulse mask. This 5-bit value defines the level of the analog voltage in steps of about 80mV (for this sub pulse) on transmit pins of the E1T1 span.

  OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS  = OTX_ATTR_END + 4,  // @emem (OTX_ATTRIB_BOOL) Read/Write. If set to OTX_TRUE, the ABCD bit devices 
                                                                  // (OTX_LDEVICE_T1_ROB_BIT_ACCESS and OTX_LDEVICE_E1_ABCD_BIT_ACCESS) will report changes 
                                                                  // in the ABCD bit code on a channelized basis (using the OTX_T1E1_EC_BITROB_SIGNALING event). 
                                                                  // The <e OtxEventDataS.m_nParam> field in the OTX_T1E1_EC_BITROB_SIGNALING event will contain
                                                                  // channel number in bit12-bit8 and the ABCD bit code in bit3-bit0 (A=bit3,B=bit2,C=bit1,D=bit0)
                                                                  // <nl> The default is OTX_FALSE and OTX_LDEVICE_E1_ABCD_BIT_ACCESS event then contains ABCD bit
                                                                  // information as described by <t OtxE1ABCDBitDataS> and <t OtxT1RobBitDataS>.

  OTX_ATTR_T1E1_S_BIT_CHANGE_EVENTS          = OTX_ATTR_END + 5,  // @emem (OTX_ATTRIB_BOOL) Read/Write. If set to OTX_TRUE, OTX_LDEVICE_E1_S_BIT_ACCESS will generate
                                                                  // an OTX_T1E1_EC_SA_BITS_CHANGED event for each change in the received Sa bits.

  OTX_ATTR_T1E1_SECONDS_INTERRUPT            = OTX_ATTR_END + 6,  // @emem (OTX_ATTRIB_BOOL) Read/Write. Default is OTX_FALSE. If set to OTX_TRUE, the device will generate 
                                                                  // <e OtxT1E1EventCodesE.OTX_T1E1_EC_SEC> event once every second. This event can be utilized
                                                                  // as a "heartbeat" watch dog from the device to verify that interrupts from this device are
                                                                  // operational.

  OTX_ATTR_T1E1_AUTOMATIC_RESET              = OTX_ATTR_END + 7,  // @emem (OTX_ATTRIB_BOOL) Read/Write. Default is OTX_TRUE. If set to OTX_TRUE, the device will generate 
                                                                  // monitor the contents of its register and if an unexpected discrepancy is encountered, the device will reset itself
                                                                  // and reconfigure itself to the last known good state. If the automatic reset is triggered, the device 
                                                                  // generated an <e OtxT1E1EventCodesE.OTX_T1E1_EC_AUTO_RESET>
                                                                  // In case a scenario of cyclic resets occurs, this attribute will be set to OTX_FALSE after 10 reset attempts, to avoid
                                                                  // a system lockup.

  OTX_ATTR_T1E1_RX_INPUT_THRESHOLD           = OTX_ATTR_END + 8,  // @emem (OTX_ATTRIB_UINT32) Read/Write. In millivolts.
                                                                  // No signal (<e OtxT1E1EventCodesE.OTX_T1E1_EC_LOS>) is declared if the voltage between pins RL1 and RL2 
                                                                  // (analog input pins to the T1E1 device) drops
                                                                  // below this limit (in millivolts) and the received data
                                                                  // stream has no transition for a period defined in <e OtxT1ConfigOptionsS.nLosSensitivity>.
                                                                  // Note: The threshold voltage is dependant on the value set for <e OtxT1ConfigOptionsS.bReceiveEqualizer>.
                                                                  // For further details please refer to the data sheet for the T1E1 device.

  OTX_ATTR_T1E1_TR08_MODE                    = OTX_ATTR_END + 9,  // @emem (OTX_ATTRIB_BOOL) Read/Write. If set to OTX_TRUE, the OTX_LDEVICE_T1_ROB_BIT_ACCESS
                                                                  // device will decode the received AB bits according the TR08 specification, The following 9 states
                                                                  // are reported:
                                                                  // <nl> State <en-> (A B)
                                                                  // <nl> =============  
                                                                  // <nl>  0 <en-> (0 0)
                                                                  // <nl>  1 <en-> (0 1)
                                                                  // <nl>  2 <en-> (0 0/1)
                                                                  // <nl>  3 <en-> (1 0)
                                                                  // <nl>  4 <en-> (1 1)
                                                                  // <nl>  5 <en-> (1 0/1)
                                                                  // <nl>  6 <en-> (0/1 0)
                                                                  // <nl>  7 <en-> (0/1 1)
                                                                  // <nl>  8 <en-> (0/1 0/1)
                                                                  // <nl> where "0/1" indicates toggling between 0 and 1.
                                                                  // <nl> The state is on a channelized basis (using the OTX_T1E1_EC_TR08_STATE_CHANGE event). 
                                                                  // The <e OtxEventDataS.m_nParam> field in the OTX_T1E1_EC_TR08_STATE_CHANGE event will contain
                                                                  // channel number in bit12-bit8 and the state number in bit3-bit0 
                                                                  // <nl> The default for this attribute is OTX_FALSE.
                                                                  // For this attribute to have any effect 
                                                                  // <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS> must be set OTX_TRUE.
                                                                  // This mode <e OtxT1FrameFormatE.OTX_T1_FF_F72> frame format.

  OTX_ATTR_T1E1_RX_AMPLIFIER                 = OTX_ATTR_END + 10, // @emem (OTX_ATTRIB_BOOL) Read/Write. If set to OTX_TRUE, an amplifier is switched on 
                                                                  // in the front end circuitry of the T1/E1 span.
                                                                  // <nl> This attribute is applicable to all OTX boards with a front end amplifier. I.e. it is applicable to 
                                                                  // Thor-1-PCI-Plus, Thor-2-PCI-Plus, Thor-4-PCI-Plus, Thor-8-PCI-Plus, Gimle-16-PCI-Plus, 
                                                                  // Gimle-16-PCI-Basic-32, Gimle-16-PCI-Basic-34, Thor-1-PCI-Express, Thor-2-PCI-Express, 
                                                                  // Thor-2-PCMCIA-PLUS, Thor-2-PCMCIA-EX, Thor-2-PCMCIA-PRO, Thor-2-PCMCIA-CST (and possibly later variants).
                                                                  // <nl> The T1/E1 signal amplifier is turned on by setting this attribute to OTX_TRUE. This amplifier should
                                                                  // only be used in conjunction with <f OtxT1E1SetMonitorMode> to boost a -30dB attenuated signal to a level that
                                                                  // can be monitored by the T1/E1 device. If you are monitoring a -20dB signal then this attribute should be set 
                                                                  // to OTX_FALSE, and <f OtxT1E1SetMonitorMode> should be turned on.

  OTX_ATTR_T1E1_AUTOMATIC_FORCE_RESYNC       = OTX_ATTR_END + 11, // @emem (OTX_ATTRIB_BOOL) Read/Write. Default is OTX_FALSE. If set to OTX_TRUE, the device will
                                                                  // monitor the LOF condition and will attempt to force a resynchronization if LOF is detected.
                                                                  // The LOF condition is checked once a second if the attribute OTX_ATTR_T1E1_SECONDS_INTERRUPT
                                                                  // has been set to OTX_TRUE.
                                                                  // If the automatic resync is triggered, the device will generate
                                                                  // an <e OtxT1E1EventCodesE.OTX_T1E1_EC_AUTO_FORCE_RESYNC>
                                                                  // The number of remaining attempts is returned in the event data.  When the remaining attempts
                                                                  // reaches zero, this attribute will be set to OTX_FALSE.

  OTX_ATTR_T1E1_AUTOMATIC_LOOP_CODE_RESPONSE = OTX_ATTR_END + 12, // @emem (OTX_ATTRIB_BOOL) Read/Write. Default is OTX_FALSE. If set to OTX_TRUE, the device will
                                                                  // automatically respond to loop codes detected on the span.

  OTX_ATTR_T1E1_LOOP_UP_CODE_ENABLE          = OTX_ATTR_END + 13, // @emem (OTX_ATTRIB_BOOL) Read/Write. Default is OTX_FALSE. If set to OTX_TRUE, the device will
                                                                  // keep sending the Line Loop Up Code on its T1 span.
                                                                  // <nl> Note that this attribute is mutally exclusive with 
                                                                  // the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_LOOP_DOWN_CODE_ENABLE> attribute.
                                                                  // <nl> Note that this attribute is only implemented for T1 mode yet.

  OTX_ATTR_T1E1_LOOP_DOWN_CODE_ENABLE        = OTX_ATTR_END + 14, // @emem (OTX_ATTRIB_BOOL) Read/Write. Default is OTX_FALSE. If set to OTX_TRUE, the device will
                                                                  // keep sending the Line Loop Down Code on its T1 span.
                                                                  // <nl> Note that this attribute is mutally exclusive with 
                                                                  // the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_LOOP_UP_CODE_ENABLE> attribute.
                                                                  // <nl> Note that this attribute is only implemented for T1 mode yet.


  OTX_ATTR_T1E1_LOOP_CODE_PATTERN            = OTX_ATTR_END + 15, // @emem (OTX_ATTRIB_UINT32) Read/Write. Loop Code Bit Pattern (both Up and Down Codes). Default is 0, which means standard loop codes are applied.
                                                                  // If set to a non-zero value, the following bit ranges apply:
                                                                  // <nl> bit 31-24: Number of bits in the Loop Up Code
                                                                  // <nl> bit 23-16: Bit pattern for the Loop Up Code
                                                                  // <nl> bit 15-08: Number of bits in the Loop Down Code
                                                                  // <nl> bit 07-00: Bit pattern for the Loop Down Code
                                                                  // the upper 16 specifies the number of bits in the bit
                                                                  // pattern (range is 2-8 bits). The lower 16 bits specifies the bit pattern. 
                                                                  // <nl> Common T1 Loop Codes are:
                                                                  // <nl> Up:10000    Down:100      (attribute value 0x05100304) Customer Service Unit loop code.
                                                                  // <nl> Up:1100     Down:1110     (attribute value 0x040C040E) In-band 4-bit facility, smart jack loop loop code.
                                                                  // <nl> Up:11000    Down:11100    (attribute value 0x0518051C) In-band 5-bit facility, smart jack loop loop code.
                                                                  // <nl> Up:100000   Down:100      (attribute value 0x06200304) In-band 6-bit facility, smart jack loop loop code.
                                                                  // <nl> Up:01110000 Down:00011100 (attribute value 0x0870081C) ESF out-of-band Line loop codes
                                                                  // <nl> Up:00101000 Down:01001100 (attribute value 0x0828084C) ESF out-of-band Payload loop codes
                                                                  // <nl> Up:01001000 Down:00100100 (attribute value 0x08480824) ESF out-of-band Network loop codes
                                                                  // 
                                                                  // <nl> Up:00001    Down:001      (attribute value 0x05010301) Customer Service Unit loop code.
                                                                  // <nl> Up:0011     Down:0111     (attribute value 0x04030407) In-band 4-bit facility, smart jack loop loop code.
                                                                  // <nl> Up:00011    Down:00111    (attribute value 0x05030507) In-band 5-bit facility, smart jack loop loop code.
                                                                  // <nl> Up:000001   Down:001      (attribute value 0x06010301) In-band 6-bit facility, smart jack loop loop code.
                                                                  // <nl> Up:00001110 Down:00111000 (attribute value 0x080E0838) ESF out-of-band Line loop codes
                                                                  // <nl> Up:00010100 Down:00110010 (attribute value 0x08140832) ESF out-of-band Payload loop codes
                                                                  // <nl> Up:00010010 Down:00100100 (attribute value 0x08120824) ESF out-of-band Network loop codes
                                                                  // <nl> Note: Bit0 of the loop code bit pattern is transmitted last
                                                                  // <nl> Note that this attribute is only implemented for T1 mode yet.

  OTX_ATTR_T1E1_TX_IDLE_CHARACTER            = OTX_ATTR_END + 16, // @emem (OTX_ATTRIB_UINT32) Read/Write. Character transmitted when a timeslot is in an idle state.
                                                                  // This attribute sets a user defined idle character to be used when a call to OtxSetAttribute with OTX_ATTR_T1E1_USE_TX_IDLE_CHARACTER
                                                                  // is set to OTX_TRUE.

  OTX_ATTR_T1E1_USE_TX_IDLE_CHARACTER        = OTX_ATTR_END + 17, // @emem (OTX_ATTRIB_BOOL) Read/Write. Flag to use a previously supplied idle character (see OTX_ATTR_T1E1_TX_IDLE_CHARACTER).
                                                                  // When this attribute is called with a value of OTX_TRUE, then the user supplied idle character supplied with a previous call
                                                                  // to OtxSetAttribute with OTX_ATTR_T1E1_TX_IDLE_CHARACTER, is set as the idle character.  When set to false, if the Line Mode
                                                                  // has been set, tnen the internal default idle character is used.

  OTX_ATTR_T1E1_LINE_VOLTAGE                 = OTX_ATTR_END + 18, // @emem (OTX_ATTRIB_UINT32) Read only. Returns the T1/E1 span line voltage in millivolts (peak voltage).
                                                                  // <nl>. The voltage range for the voltage meter is 0 - 4 Volts.
                                                                  // <nl>. The accuracy is +/- 100mV.

  OTX_ATTR_T1E1_LINE_SLIP_CNT_POSITIVE       = OTX_ATTR_END + 19, // @emem (OTX_ATTRIB_UINT32) Read/Write. Returns the number of positive slips received.
                                                                  // <nl>. Positive slip count is cleared on write.

  OTX_ATTR_T1E1_LINE_SLIP_CNT_NEGATIVE       = OTX_ATTR_END + 20, // @emem (OTX_ATTRIB_UINT32) Read/Write. Returns the number of negative slips received.
                                                                  // <nl>. Positive slip count is cleared on write.

  OTX_ATTR_T1E1_END                          = OTX_ATTR_END + 21
};

typedef enum OtxT1E1AttributesE OtxT1E1AttributesE;




/*******************************************************************************
  @enum
    Result Codes

*******************************************************************************/
enum OtxT1E1ResultCodesE {
  OTX_T1E1_RES_BAD_AIS_DETECTION_OPT    = OTX_RES_END +  0 ,  // @emem Invalid parameter value passed for AIS Detection.        
  OTX_T1E1_RES_BAD_ALARM_TYPE           = OTX_RES_END +  1 ,  // @emem Invalid parameter value passed for Alarm Type.           
  OTX_T1E1_RES_BAD_CLOCK_MODE           = OTX_RES_END +  2 ,  // @emem Invalid parameter value passed for Clock Mode.           
  OTX_T1E1_RES_BAD_FRAME_FORMAT         = OTX_RES_END +  3 ,  // @emem Invalid parameter value passed for Frame Format.         
  OTX_T1E1_RES_BAD_HDB3_ERROR_OPT       = OTX_RES_END +  4 ,  // @emem Invalid parameter value passed for HDB3 Error Type.      
  OTX_T1E1_RES_BAD_LOOP_TYPE            = OTX_RES_END +  5 ,  // @emem Invalid parameter value passed for Loop Mode.            
  OTX_T1E1_RES_BAD_REGAIN_M_FRAME_OPT   = OTX_RES_END +  6 ,  // @emem Invalid parameter value passed for Regain Multiframe.    
  OTX_T1E1_RES_BAD_REMOTE_ALARM_OPT     = OTX_RES_END +  7 ,  // @emem Invalid parameter value passed for Remote Alarm.         
  OTX_T1E1_RES_BAD_RESYNC_OPT           = OTX_RES_END +  8 ,  // @emem Invalid parameter value passed for Resynchronization.    
  OTX_T1E1_RES_BAD_RX_EQUALIZER_OPT     = OTX_RES_END +  9 ,  // @emem Invalid parameter value passed for Receiver Equalizer.   
  OTX_T1E1_RES_BAD_RX_FRAME_FORMAT      = OTX_RES_END + 10 ,  // @emem Invalid parameter value passed for Receiver Frame Format.
  OTX_T1E1_RES_BAD_RX_LINE_CODE         = OTX_RES_END + 11 ,  // @emem Invalid parameter value passed for Reciever Line Code.   
  OTX_T1E1_RES_BAD_RX_REM_ALRM_FORMAT   = OTX_RES_END + 12 ,  // @emem Invalid parameter value passed for Receiver Remote Alarm.
  OTX_T1E1_RES_BAD_SIGNALING_MODE       = OTX_RES_END + 13 ,  // @emem Invalid parameter value passed for Signalling Mode.      
  OTX_T1E1_RES_BAD_TX_FRAME_FORMAT      = OTX_RES_END + 14 ,  // @emem Invalid parameter value passed for Transmit Frame Format.
  OTX_T1E1_RES_BAD_TX_LINE_CODE         = OTX_RES_END + 15  , // @emem Invalid parameter value passed for Transmit Line Code.   
  OTX_T1E1_RES_BAD_TX_POWER_OPT         = OTX_RES_END + 16  , // @emem Invalid parameter value passed for Transmit Power.       
  OTX_T1E1_RES_BAD_TX_REM_ALRM_FORMAT   = OTX_RES_END + 17  , // @emem Invalid parameter value passed for Transmit Remote Alarm.
  OTX_T1E1_RES_L1_DOWN                  = OTX_RES_END + 18  , // @emem Loss Of Signal. Layer 1 is down.                         
  OTX_T1E1_RES_L1_OK                    = OTX_RES_END + 19  , // @emem E1/T1 receiver is receiving valid signals. Layer 1 is OK.
  OTX_T1E1_RES_NOT_SETUP                = OTX_RES_END + 20  , // @emem The device is not setup to support this action.          
  OTX_T1E1_RES_TX_BUSY                  = OTX_RES_END + 21  , // @emem Transmitter is busy sending a frame.                     
  OTX_T1E1_RES_WRONG_CONTEXT            = OTX_RES_END + 22  , // @emem Wrong context for this action.                           
  OTX_T1E1_RES_UNKNOWN_STATUS_CODE      = OTX_RES_END + 23  , // @emem Invalid event code passed to this function.              
  OTX_T1E1_RES_UNKNOWN_MODE             = OTX_RES_END + 24  , // @emem The device is not setup in a valid mode.                 
  OTX_T1E1_RES_INVALID_CHANNEL          = OTX_RES_END + 25  , // @emem Invalid channel.                                         
  OTX_T1E1_RES_UNSUPPORTED_VER          = OTX_RES_END + 26  , // @emem Unsupported Device version.                              
  OTX_T1E1_RES_END                      = OTX_RES_END + 27
};

typedef enum OtxT1E1ResultCodesE OtxT1E1ResultCodesE;


/*******************************************************************************
  @enum
    Event Codes

    Note that for line interfaces these messages can have a different
    meaning depending on whether the Line Interfaces (LIs) are configured to
    E1 or T1 mode.

*******************************************************************************/
enum OtxT1E1EventCodesE {
  OTX_T1E1_EC_T8MS_ISF             = 0x00, // @emem  E1: Receive Timeout 8 MSec
                                           // <nl>   T1: Incorrect Sync Format

  OTX_T1E1_EC_CASC_RSC             = 0x01, // @emem  E1: Received CAS information Changed
                                           // <nl>   T1: Received Signalling information Changed

  OTX_T1E1_EC_CRC4_CRC6            = 0x02, // @emem  E1: Receive CRC4 error
                                           // <nl>   T1: Receive CRC6 error

  OTX_T1E1_EC__CASE                = 0x03, // @emem  E1: -
                                           // <nl>   T1: Transmit CAS register Empty

  OTX_T1E1_EC_RDO                  = 0x04, // @emem  E1: Receive Data Overflow
                                           // <nl>   T1: Receive Data Overflow

  OTX_T1E1_EC_XDU                  = 0x05, // @emem  E1: Transmit Data Underrun
                                           // <nl>   T1: Transmit Data Underrun

  OTX_T1E1_EC_XLSC                 = 0x06, // @emem  E1: Transmit Line Status Change
                                           // <nl>   T1: Transmit Line Status Change

  OTX_T1E1_EC_FAR                  = 0x07, // @emem  E1: Frame Alignment Recovery
                                           // <nl>   T1: Frame Alignment Recovery

  OTX_T1E1_EC_LFA                  = 0x08, // @emem  E1: Loss of Frame Alignment
                                           // <nl>   T1: Loss of Frame Alignment

  OTX_T1E1_EC_MFAR                 = 0x09, // @emem  E1: MultiFrame Alignment Recovery
                                           // <nl>   T1: MultiFrame Alignment Recovery

  OTX_T1E1_EC_T400MS_LMFA          = 0x0A, // @emem  E1: Receive Timeout 400 MSec
                                           // <nl>   T1: Loss of MultiFrame Alignment

  OTX_T1E1_EC_AIS                  = 0x0B, // @emem  E1: Alarm Indication Signal (Blue Alarm)
                                           // <nl>   T1: Alarm Indication Signal (Blue Alarm)

  OTX_T1E1_EC_LOS                  = 0x0C, // @emem  E1: Loss Of Signal (Red Alarm)
                                           // <nl>   T1: Loss Of Signal (Red Alarm)

  OTX_T1E1_EC_RAR                  = 0x0D, // @emem  E1: Remote Alarm Recovery
                                           // <nl>   T1: Remote Alarm Recovery

  OTX_T1E1_EC_RA                   = 0x0E, // @emem  E1: Remote Alarm (Yellow Alarm)
                                           // <nl>   T1: Remote Alarm (Yellow Alarm)

  OTX_T1E1_EC_LMFA16_XSLP          = 0x0F, // @emem  E1: Loss of MultiFrame Alignment ts16
                                           // <nl>   T1: Transmit SLiP indication

  OTX_T1E1_EC_AIS16_               = 0x10, // @emem  E1: Alarm Indication Signal ts16 status change
                                           // <nl>   T1: -

  OTX_T1E1_EC_RA16_LLBSC           = 0x11, // @emem  E1: Remote Alarm ts16 status change
                                           // <nl>   T1: Line Loop Back Status Change

  OTX_T1E1_EC_API_                 = 0x12, // @emem  E1: Auxuillary Pattern Indication
                                           // <nl>   T1: -

  OTX_T1E1_EC_SLN                  = 0x13, // @emem  E1: Receive Slip Negative
                                           // <nl>   T1: Receive Slip Negative

  OTX_T1E1_EC_SLP                  = 0x14, // @emem  E1: Receive Slip Positive
                                           // <nl>   T1: Receive Slip Positive

  OTX_T1E1_EC_ACTIVE               = 0x15, // @emem  E1: Physical link ACTIVE indication
                                           // <nl>   T1: Physical link ACTIVE indication

  OTX_T1E1_EC_DEACTIVE             = 0x16, // @emem  E1: Physical link DEACTIVE indication
                                           // <nl>   T1: Physical link DEACTIVE indication

  OTX_T1E1_EC_XLS                  = 0x17, // @emem  E1: Transmit Line Short
                                           // <nl>   T1: Transmit Line Short

  OTX_T1E1_EC_XLO                  = 0x18, // @emem  E1: Transmit Line Open
                                           // <nl>   T1: Transmit Line Open

  OTX_T1E1_EC_XPR                  = 0x19, // @emem  Transmit Pool Ready

  OTX_T1E1_EC_SEC                  = 0x1a, // @emem  Second

  OTX_T1E1_EC_ES                   = 0x1b, // @emem  Errored second

  OTX_T1E1_EC_RME                  = 0x1c, // @emem  Receive Message End

  OTX_T1E1_EC_RPF                  = 0x1d, // @emem  Receive Pool Full

  OTX_T1E1_EC_XSP                  = 0x1e, // @emem  E1: Transmit Slip Positive
                                           // <nl>   T1: Transmit Slip Positive

  OTX_T1E1_EC_XSN                  = 0x1f, // @emem  E1: Transmit Slip Negative
                                           // <nl>   T1: Transmit Slip Negative

  OTX_T1E1_EC_BITROB_SIGNALING     = 0x20, // @emem  E1: -
                                           // <nl>   T1: BitRob Signaling Data

  OTX_T1E1_EC_FEC                  = 0x21, // @emem  E1: Frame Error Count
                                           // <nl>   T1: Frame Error Count

  OTX_T1E1_EC_CVC                  = 0x22, // @emem  E1: Code Violation Count
                                           // <nl>   T1: Code Violation Count

  OTX_T1E1_EC_EBC                  = 0x23, // @emem  E1: E-bit Error Count
                                           // <nl>   T1: Errored Block Count

  OTX_T1E1_EC_CEC1                 = 0x24, // @emem  E1: CRC Error Count
                                           // <nl>   T1: CRC Error Count

  OTX_T1E1_EC_CEC2                 = 0x25, // @emem  E1: CRC Error Count (reported from TE via Sa6-bit)
                                           // <nl>   T1: -

  OTX_T1E1_EC_CEC3                 = 0x26, // @emem  E1: CRC Error Count (detected at T reference point via Sa6-bit)
                                           // <nl>   T1: -

  OTX_T1E1_EC_APIR                 = 0x27, // @emem  E1: Auxiliary Pattern Indication (Blue Alarm) Recovery
                                           // <nl>   T1: -

  OTX_T1E1_EC_LOSR                 = 0x28, // @emem  E1: Loss Of Signal (Red Alarm) Recovery
                                           // <nl>   T1: Loss Of Signal (Red Alarm) Recovery

  OTX_T1E1_EC_LMFAR16              = 0x29, // @emem  E1: Loss of MultiFrame Alignment ts16 Recovery
                                           // <nl>   T1: -

  OTX_T1E1_EC_AISR                 = 0x2a, // @emem  E1: Alarm Indication Signal (Blue Alarm) Recovery
                                           // <nl>   T1: Alarm Indication Signal (Blue Alarm) Recovery

  OTX_T1E1_EC_SA_BITS_CHANGED      = 0x2b, // @emem  E1: SA-Bits (4-8) changed. Bit0 is Sa bit from Frame1, Bit0 is Sa bit from Frame3, ... , Bit7 is Sa bit from Frame15. Bit11-Bit8 contains the Sa number (4-8).
                                           // <nl>   T1: -

  OTX_T1E1_EC_DL_BITS_RECEIVED     = 0x2c, // @emem  E1: -
                                           // <nl>   T1: DL-Bits Received:
                                           // <nl> Mode <e OtxT1FrameFormatE.OTX_T1_FF_F12>:
                                           // <nl> Bit0 is received in frame 2 and Bit7 in frame 12.
                                           // <nl> Remaining bits are zero.
                                           // <nl> 
                                           // <nl> Mode <e OtxT1FrameFormatE.OTX_T1_FF_ESF>:
                                           // <nl> Bit0 is received in frame  1 and Bit7  in frame 15.
                                           // <nl> Bit8 is received in frame 17 and Bit15 in frame 23.
                                           // <nl> Remaining bits are zero.
                                           // <nl> 
                                           // <nl> Mode <e OtxT1FrameFormatE.OTX_T1_FF_F72>:
                                           // <nl> Bit0  is received in frame 26 and Bit7  in frame 40
                                           // <nl> Bit8  is received in frame 42 and Bit15 in frame 56.
                                           // <nl> Bit16 is received in frame 58 and Bit23 in frame 72.
                                           // <nl> Remaining bits are zero.

  OTX_T1E1_EC_DL_BITS_SENT         = 0x2d, // @emem  E1: -
                                           // <nl>   T1: DL-Bits Sent. The function <f OtxT1SetDlBitData> needs to
                                           // be called to provide more DL-bits to be transmitted by the T1E1 device.
                                           // If <f OtxT1SetDlBitData> is not called before the start of the next
                                           // multiframe then the DL-Bit device will continue to transmit the same DL-bits 
                                           // (as were provided by the most recent call to <f OtxT1SetDlBitData>.
                                           // The bits should be formatted in the same way as 
                                           // in <e OtxT1E1EventCodesE.OTX_T1E1_EC_DL_BITS_RECEIVED>.

  OTX_T1E1_EC_AUTO_DISABLED        = 0x2e, // @emem  E1: Fatal error has occurred. The device is automatically disabled.
                                           // <nl>       Try reviving the device with <f OtxDrvReset> and <f OtxDrvEnable>.
                                           // <nl>   T1: Fatal error has occurred. The device is automatically disabled.
                                           // <nl>       Try reviving the device with <f OtxDrvReset> and <f OtxDrvEnable>.

  OTX_T1E1_EC_AUTO_RESET           = 0x2f, // @emem  E1: Serious error has occurred. The device has performed an automatic self-reset.
                                           // @emem  T1: Serious error has occurred. The device has performed an automatic self-reset.

  OTX_T1E1_EC_TR08_STATE_CHANGE    = 0x30, // @emem  E1: N/A
                                           // @emem  T1: TR08 State change occurred. Please refer to OTX_ATTR_T1E1_TR08_MODE.
                                           //

/**************************/

//#ifdef _TURNSTONE_

/**************************/

  OTX_T1E1_EC_QRSS_PAT_SYNC        = 0x31, // @emem Internal use
  
  OTX_T1E1_EC_QRSS_PAT_ASYNC       = 0x32, // @emem Internal use

  OTX_T1E1_EC_BITERROR_UPDATE      = 0x33, // @emem Internal use

  OTX_T1E1_EC_RX_LEVEL_UPDATE      = 0x34, // @emem Internal use

  OTX_T1E1_EC_RX_EQ_ADAPTING       = 0x35, // @emem Internal use

  OTX_T1E1_EC_RX_EQ_NOISE          = 0x36, // @emem Internal use 

  OTX_T1E1_EC_RX_EQ_INVALID        = 0x37, // @emem Internal use

  OTX_T1E1_EC_WEAKSIGNAL           = 0x38, // @emem Internal use

  OTX_T1E1_EC_PULSE_DETECTION_COMPLETE = 0x39, // @emem Internal use

  OTX_T1E1_EC_FF_SCAN_COMPLETE = 0x3a, // @emem Internal use

  OTX_T1E1_EC_PULSEDENSITY_VIOLATION_ON = 0x3b, // @emem Internal use 

  OTX_T1E1_EC_PULSEDENSITY_VIOLATION_OFF = 0x3c, // @emem Internal use 

  OTX_T1E1_EC_EXCESSZERO_ON = 0x3d, // @emem Internal use 

  OTX_T1E1_EC_EXCESSZERO_OFF = 0x3e, // @emem Internal use

//**************************

//#endif  // _TURNSTONE_

//**************************

  OTX_T1E1_EC_AUTO_FORCE_RESYNC    = 0x3f, // @emem  E1: LOF detected and OTX_ATTR_T1E1_AUTOMATIC_FORCE_RESYNC is set.  The device attempted to perform an automatic forced resynchronization.
                                           // @emem  T1: LOF detected and OTX_ATTR_T1E1_AUTOMATIC_FORCE_RESYNC is set.  The device attempted to perform an automatic forced resynchronization.

  OTX_T1E1_EC_LOOP_UP_CODE         = 0x40, // @emem  E1: Loop Up Code (Activate) detected.
                                           // @emem  T1: Loop Up Code (Activate)  detected.

  OTX_T1E1_EC_LOOP_DOWN_CODE       = 0x41, // @emem  E1: Loop Down Code (Deactivate) detected.
                                           // @emem  T1: Loop Down Code (Deactivate) detected.

  OTX_T1E1_EC_END_OF_EVENT_CODES   // @emem Internal use

};

typedef enum OtxT1E1EventCodesE OtxT1E1EventCodesE;



/*----------------------------------------------------------------------------*\
  T1/E1 Common Types
\*----------------------------------------------------------------------------*/

/*******************************************************************************

  @enum
    Operating modes for the T1/E1 Line Interface (Li) Devices

  @xref
    <f OtxT1E1SetLiMode>

*******************************************************************************/
enum OtxT1E1LiModeE {
  OTX_T1E1_MODE_NO = 0,    // @emem The line interface has not been configured
  OTX_T1E1_MODE_T1 = 1,    // @emem The line interface is configured to operate in T1 mode.
  OTX_T1E1_MODE_E1 = 2     // @emem The line interface is configured to operate in E1 mode.
};

typedef enum OtxT1E1LiModeE OtxT1E1LiModeE;


/*******************************************************************************

  @enum
    Layer 1 Status

  @xref
    <f OtxT1E1GetLineStatus>

*******************************************************************************/
enum OtxT1E1Layer1StatusE {
  OTX_T1E1_LAYER1_DOWN = 0,  // @emem Loss of Signal (LOS) or Loss Of Frame Alignment (LFA) is present
  OTX_T1E1_LAYER1_OK   = 1   // @emem E1/T1 signal present (No Loss of Signal and no Loss Of Frame Alignment) 
};

typedef enum OtxT1E1Layer1StatusE OtxT1E1Layer1StatusE;


/*******************************************************************************

  @enum
    Clock mode 

*******************************************************************************/
enum OtxT1E1ClkModeE {
  OTX_T1E1_CLOCK_NO = 0,       // No Clock mode has been set
  OTX_T1E1_CLOCK_MASTER = 1,   // Clock Master
  OTX_T1E1_CLOCK_SLAVE = 2     // Clock Slave
};

typedef enum OtxT1E1ClkModeE OtxT1E1ClkModeE;


/*******************************************************************************

  @enum
    Alarms types that can be send towards the Remote End

  @xref
    <f OtxT1E1AlarmOn>
    <f OtxT1E1AlarmOff>

*******************************************************************************/
enum OtxT1E1AlarmTypeE {
  OTX_T1E1_ALARM_AUXP       = 0,  // @emem Alarm Type Auxilliary Pattern (AUXP)
  OTX_T1E1_ALARM_AIS        = 1,  // @emem Alarm Indication Signal (AIS, Blue Alarm)
  OTX_T1E1_ALARM_RAI        = 2,  // @emem Remote Alarm Indication (RAI, Yellow Alarm) 
  OTX_T1E1_ALARM_SIMULATION = 3,  // @emem Initiates internal error simulation of AIS,
                                  //       loss of signal, loss of synchronization,
                                  //       auxilliary pattern indication, slip, framing
                                  //       errors, CRC errors, and code violations.
};

typedef enum OtxT1E1AlarmTypeE OtxT1E1AlarmTypeE;


/*******************************************************************************

  @enum
    Line Interface loop type

  @xref
    <f OtxT1E1SetLineLoop>

*******************************************************************************/
enum OtxT1E1LoopE {
  OTX_T1E1_LOOPS_OFF                    = 0,   // @emem All Line Loops off
  OTX_T1E1_LINE_LOOP                    = 1,   // @emem Line Loop (Payload Loopback of the external highway). Looped data will pass through the entire receiver including the wander and jitter compensation.
  OTX_T1E1_REMOTE_ON_LOOP_JITTER_ATTEN  = 2,   // @emem Remote Loop with Jitter Attenuation On (Loopback of the external highway bypassing the framer)
  OTX_T1E1_REMOTE_OFF_LOOP_JITTER_ATTEN = 3,   // @emem Remote Loop with Jitter Attenuation Off (Loopback of the external highway bypassing the framer)
  OTX_T1E1_LOCAL_LOOP                   = 4,   // @emem Local Loop. The receiver on the external highway is disconnected. The internal highway is looped back to the time-space switch. 
};

typedef enum OtxT1E1LoopE OtxT1E1LoopE;


/*******************************************************************************
   @enum
     line termination for an E1/T1 span.
   
   @comm
     line termination mode should match the type of cable that is connected to 
     the Line Interface. If the Line Interface is used for monitoring and 
     connected in a bridged configuration with short monitor tap cables, the 
     High Impedance termination mode should be used. If the Line Interface is 
     used for generation, or the  board is connected to a -20dB monitor port, 
     then one of the 75 (E1), 100 (T1), or 120 (E1) ohm modes should be used. 

     Setting the termination mode also affects transmit pulse level. The level is
     set to comply with the G.703 and T1.102 standards. I.e.

     <nl>G.703 E1 120 ohm Balanced:   Amplitude=3.0V  (+/- 0.3V)
     <nl>G.703 T1 100 ohm Balanced:   Amplitude=3.0V  (+/- 0.6V)
     <nl>G.703 E1  75 ohm Unbalanced: Amplitude=2.37V (+/- 0.237V)
     
     The default is high impedance (no termination), which is set when the 
     driver loads.

  @xref
     <f OtxT1E1SetLineTermination>

*******************************************************************************/
enum OtxT1E1TermModeE {
  OTX_T1E1_TERM_120_OHM,        // @emem Span is terminated for 120 ohm twisted pair (balanced differential signal)
  OTX_T1E1_TERM_100_OHM,        // @emem Span is terminated for 100 ohm twisted pair (balanced differential signal)
  OTX_T1E1_TERM_75_OHM,         // @emem Span is terminated for 75 ohm coax (unbalanced unipolar signal)
  OTX_T1E1_TERM_HIGHZ,          // @emem Span is not terminated (Balanced, High impedance, for monitoring scenarios when short (<lt>2 meter) monitor taps are used)
  OTX_T1E1_TERM_75_OHM_BALANCED // @emem Span is terminated for 75 ohm balanced differential signal (for special monitoring conditions)
};

typedef enum OtxT1E1TermModeE OtxT1E1TermModeE;


/*******************************************************************************
   @enum
     Single Bit Defect types
   
   @comm
     A single bit defect can be inserted into the T1/E1 bit stream. This type
     describes the different types of defects (errors) that can be inserted.

  @xref
     <f OtxT1E1InsertSingleBitDefect>

*******************************************************************************/
enum OtxT1E1DefectE {
  OTX_T1E1_DEFECT_FAS,        // @emem FAS (Frame Alignment Sequence) defect
  OTX_T1E1_DEFECT_MULTIFRAME, // @emem Multiframe defect
  OTX_T1E1_DEFECT_CRC,        // @emem CRC defect
  OTX_T1E1_DEFECT_CAS,        // @emem CAS defect
  OTX_T1E1_DEFECT_PRBS,       // @emem Pseudo Random Binary Sequence defect. Only applicable in the device is configured in PRBS mode.
  OTX_T1E1_DEFECT_BPV         // @emem Bipolar violation defect
};

typedef enum OtxT1E1DefectE OtxT1E1DefectE;


/*******************************************************************************

  @struct
    Specific details of the current Line status. This structure is returned by 
    the function <f OtxT1E1GetLayer1StatusEx>. 

  @xref
    <f OtxT1E1GetLayer1StatusEx>

*******************************************************************************/
struct OtxT1E1Layer1StatusExS {
  OTX_BOOL bRedAlarm;         // @field OTX_TRUE is the received signal level is below the Receive Threshold. (a.k.a Red Alarm)
  OTX_BOOL bBlueAlarm;        // @field OTX_TRUE if the device has detected an Alarm Indication Signal (all ones) on the span.
  OTX_BOOL bYellowAlarm;      // @field OTX_TRIE if the device has detected a Remote Alarm
  OTX_BOOL bFrameError;       // @field OTX_TRUE if the device can not find basic framing on the received span
  OTX_BOOL bMultiFrameError;  // @field OTX_TRUE if the device can not find a good multiframing candidate.
};

typedef struct OtxT1E1Layer1StatusExS OtxT1E1Layer1StatusExS;


/*******************************************************************************

  @struct
    Specific details of the current Line status. This structure is returned by 
    the function <f OtxT1E1GetLiMode>. 

  @xref
    <f OtxT1E1GetLiMode>

*******************************************************************************/
struct OtxT1E1GetLiModeS {
  OtxT1E1LiModeE   eT1E1Mode;          // @field <t OtxT1E1LiModeE> operating modes for the T1/E1 Line Interface (Li) Devices
  OtxT1E1TermModeE eT1E1TermMode;      // @field <t OtxT1E1TermModeE> line termination for an E1/T1 span
  OTX_BOOL         bTxPinEnabled;      // @field State of the Transmit pair of the Line Interface <t OtxT1E1XmitPinControl>
  OTX_BOOL         bMonitorModeActive; // @field If OTX_TRUE the receiver sensitivity is increased to detect resistively attenuated signals of -20 dB
};

typedef struct OtxT1E1GetLiModeS OtxT1E1GetLiModeS;




/*----------------------------------------------------------------------------*\
  T1 Specfic Configuration Types
\*----------------------------------------------------------------------------*/

/*******************************************************************************

  @enum
     T1 Signalling Modes

  @xref
     <t OtxT1ConfigOptionsS>
     <f OtxT1SetConfiguration>

*******************************************************************************/
enum OtxT1SignallingModeE {
  OTX_T1_SM_CCS      = 0,    // @emem Common Channel Signalling
  OTX_T1_SM_CAS_CC   = 1,    // @emem Channel Associated Signalling (Common channel)
  OTX_T1_SM_CAS_BR   = 2     // @emem Channel Associated Signalling (Bit Robbing)
};

typedef enum OtxT1SignallingModeE OtxT1SignallingModeE;


/*******************************************************************************

  @enum
     T1 Line Codes

  @xref
     <t OtxT1ConfigOptionsS>
     <f OtxT1SetConfiguration>

*******************************************************************************/
enum OtxT1LineCodeE {
  OTX_T1_LC_AMI   = 0,   // @emem Alternative Mark Inversion (AMI) Line Coding
  OTX_T1_LC_B8ZS  = 1    // @emem Bipolar 8 Zero Subsititution (B8ZS) Line Coding
};

typedef enum OtxT1LineCodeE OtxT1LineCodeE;


/*******************************************************************************

  @enum
     T1 Frame Formats

  @xref
     <t OtxT1ConfigOptionsS>
     <f OtxT1SetConfiguration>

*******************************************************************************/
enum OtxT1FrameFormatE {
  OTX_T1_FF_F12      = 0,  // @emem 12-frame multiframe format (F12, D3/4, SF)
  OTX_T1_FF_F4       = 1,  // @emem 4-frame multiframe format (F4)
  OTX_T1_FF_ESF      = 2,  // @emem 24-frame multiframe format (F24, ESF)
  OTX_T1_FF_F72      = 3,  // @emem 72-frame multiframe format (F72, SLC96, remote switch mode)
  OTX_T1_FF_F12_J1   = 4,  // @emem J1 variant of 12-frame multiframe format (F12, D3/4, SF)
  OTX_T1_FF_F4_J1    = 5,  // @emem J1 variant of 4-frame multiframe format (F4)
  OTX_T1_FF_ESF_J1   = 6,  // @emem J1 variant of 24-frame multiframe format (F24, ESF)
  OTX_T1_FF_F72_J1   = 7,  // @emem J1 variant of 72-frame multiframe format (F72, SLC96, remote switch mode)
  OTX_T1_FF_UNFRAMED = 8   // @emem Unframed (Transparent) mode. In this mode the DL-bit is provided to the T1E1 device via one bit in Timeslot 0 of the highway to this device.
};

typedef enum OtxT1FrameFormatE OtxT1FrameFormatE;


/*******************************************************************************

  @enum
     Set the condition when the Loss of Frame Alignment (LFA) is reported.
  
  @comm
     LFA can be reported if 2 out of 4, 5, or 6 framing bits have been detected 
     to be incorrect.
 
  @xref
     <t OtxT1ConfigOptionsS>
     <f OtxT1SetConfiguration>

*******************************************************************************/
enum OtxT1LfaSensitivityE {
  OTX_T1_LFA_2_OUT_OF_4 = 0x00,    // @emem 2 out of 4 framing bits incorrect
  OTX_T1_LFA_2_OUT_OF_5 = 0x08,    // @emem 3 out of 4 framing bits incorrect
  OTX_T1_LFA_2_OUT_OF_6 = 0x10,    // @emem 4 out of 4 framing bits incorrect
};

typedef enum OtxT1LfaSensitivityE OtxT1LfaSensitivityE;


/*******************************************************************************

  @enum
     Select the Remote Alarm Indication (Yellow Alarm) format to be expected
     in Receive direction with the F12 and ESF Frames.

  @comm
     If Format A is selected, with F12 Frame Format the bit #2 in every channel 
     is set to `0'. With ESF Frame Format a pattern on `1111 1111 0000 0000 ...' 
     is sent in the data link channel. If Format B is selected, with F12 Frame 
     Format the Frame Synchronization (FS) bit of frame 12 is set to `0'. 
     With ESF Frame Format the bit position 2 in every time-slot is set to `0'. 
     With F4 and F72 frames formats the Yellow alarm is indicated in both 
     formats by forcing the bit position 2 to `1' in every speech channel.

  @xref
     <t OtxT1ConfigOptionsS>
     <f OtxT1SetConfiguration>

*******************************************************************************/
enum OtxT1RemoteAlarmFormatE {
  OTX_T1_RA_YELLOW_A = 0,  // @emem F12: bit2 = 0  in every channel
                           // <nl> ESF: pattern '1111 1111 0000 0000' in data link channel
  OTX_T1_RA_YELLOW_B = 1,  // @emem F12: FS bit of frame 12
                           // <nl> ESF: bit2 = 0  in every channel
};

typedef enum OtxT1RemoteAlarmFormatE OtxT1RemoteAlarmFormatE;


/*******************************************************************************

  @struct
    T1 Line Interface Configuration Options

  @xref
   <f OtxT1SetConfiguration>
   <f OtxT1GetConfiguration>

*******************************************************************************/
struct OtxT1ConfigOptionsS {
  OtxT1SignallingModeE    eSignalingMode;  // @field Signalling Mode. See type <t OtxT1SignallingModeE>. 
                                           // <nl>This setting is only used for boards equipped with the PEB2255 Line Interface device.
                                           // For boards equipped with the PEB2256 (Falc56) or the PEB22554 (QuadFalc) the signalling mode
                                           // is controlled by the <e OtxT1ConfigOptionsS.bfClearChannels> setting alone.
  OtxT1LineCodeE          eTxLineCode;     // @field Line Coding in Transmit Direction. See type <t OtxT1LineCodeE>
  OtxT1LineCodeE          eRxLineCode;     // @field Line Coding in Receive Direction
  OtxT1FrameFormatE       eFrameFormat;    // @field Frame Format. See type <t OtxT1FrameFormatE>
  OTX_BOOL                bCRC6Enable;     // @field Set whether CRC6 checking and generation will be enabled when using
                                           // the ESF frame format. When disabled, the receive will not perform the
                                           // CRC6 check and the CRC6 bits are set to `1' to the transmit direction.
  OtxT1RemoteAlarmFormatE eTxRemoteAlarmFormat; // @field Transmit Remote Alarm Format. See type <t OtxT1RemoteAlarmFormatE>
  OtxT1RemoteAlarmFormatE eRxRemoteAlarmFormat; // @field Receive Remote Alarm Format. See type <t OtxT1RemoteAlarmFormatE>
  OTX_BOOL                bAutoResync;     // @field If OTX_TRUE, the Framer will automatically re-synchronize after
                                           // loss of synchronization. If OTX_FALSE the Framer will not synchronize
                                           // automatically but a re-synchronization procedure must be specifically
                                           // initiated by the software.
  OTX_BOOL                bAutoCrcBitInversion; // @field Valid only for ESF frame format. For other frame format this value
                                           // is ignored.
                                           // <nl>If OTX_TRUE, all CRC6 bits of one outgoing sub-multiframe are
                                           // inverted in case a CRC6 error is flagged for the previous received 
                                           // sub-multiframe.
  OtxT1LfaSensitivityE    eLfaSensitivity; // @field Loss of Frame Alignment Sensitivity. See type <t OtxT1LfaSensitivityE>
  OTX_BOOL                bAutoRemoteAlarm;// @field If OTX_TRUE, a Remote Alarm (Yellow Alarm) will be sent automatically
                                           // in the outgoing data stream when the receiver is out of synchronization.
                                           // The alarm will also be automatically removed when the receiver returns to
                                           // the synchronized state.
  OTX_UINT8               nLosSensitivity; // @field Sets the sensitivity level for the Loss Of Signal (LOS) detection.
                                           // A LOS alarm will be generated if the incoming data stream has no
                                           // transitions during programmable number (N) of consecutive pulse periods.
                                           // The number given as the LOS Sensitivity value will be multiplied by 16 to
                                           // arrive to the pulse count number N. The time for detecting LOS will be
                                           // 16 x LOS Sensitivity x Pulse Length (488ns).
                                           // The maximum time is 16 x 256 x 488ns = 2ms
                                           // <nl> Possible Values: 1-256
                                           // <nl> Example Value:
                                           // LOS Sensitivity  =  128     (16 x 128 x 488ns = 1ms)
  OTX_UINT8               nLosRecovery;    // @field Sets the sensitivity level for the Loss Of Signal (LOS) recovery.
                                           // A LOS alarm will be cleared if in the incoming data a number of Pulses (N)
                                           // occur during the time period set with the LOS Recovery parameter. The time
                                           // interval starts with the first detected pulse transition and a pulse
                                           // count is incremented with every received pulse. If the pulse number is
                                           // greater than the number N set with this parameter, the LOS alarm will be cleared.
                                           // <nl> Possible Values: 1-256
  OTX_UINT16              nLineLength;     // @field Specify the length of the T1 line cable in meters. This information
                                           // is used by the T1 transceivers to adjust the transmit pulse masks according
                                           // to the line length.
                                           // <nl> Possible Values: 0-65535
  OTX_BOOL                bHighTxPower;    // @field When OTX_TRUE the pulses on the line interface will be sent with
                                           // high power. If OTX_FALSE the pulses will be sent with low power.
  OTX_BOOL                bReceiveEqualizer;// @field The Receive Equalizer can be used match the 18dB ITU-T I.431
                                           // requirement. When the Equalizer is turned off (OTX_FALSE), the Receiver is 6dB, when
                                           // turned on, the Receiver is 18dB.
  OTX_UINT32              bfClearChannels; // @field If bit robbed signalling is used (e.g. <e OtxPhysicalDeviceTypeE.OTX_LDEVICE_T1_ROB_BIT_ACCESS>) 
                                           // and if a specific channel needs to use all 8 bits of a timeslot
                                           // (e.g. a 64 kbps HDLC channel), then this channel needs to be designated as
                                           // a "clear channel".
                                           // A channel designated as a clear channel is not overwritten by internal or external bit robbing
                                           // and ZCS information. Transmission of channel assigned signaling and 
                                           // control of pulse density must applied by the user.
                                           // This field marks which channels are clear channels, and
                                           // which channels are robbed bit channels. 
                                           // <nl>The value in this parameter contains a bit pattern, where a '1' indicates a
                                           // clear channel. The least significant bit corresponds to Time-Slot 0 (TS0),
                                           // bit 1 to TS1, etc.
                                           // <nl>To define no channels as clear channels set this value to 0x000000.
                                           // <nl>To define only TS24 as a clear channel set this value to 0x800000.
};

typedef struct OtxT1ConfigOptionsS OtxT1ConfigOptionsS;


/*******************************************************************************

  @struct
    T1 Bit robbing data to be transmitted (used in CAS). See
    the <f OtxT1SetRobBitData> function.

  @comm
    Note that the A and B channels are used in frame formats F12, ESF,
    and F72. The C and D channels are only used in the ESF frame
    format.
    The struct holds 24 bits for each channel; The least significant
    bit is transmitted first (in channel 1, frame 1). This 24 bit
    value will be repeated unless <f OtxT1SetRobBitData> is called again
    with a different values in this struct.

  @xref
    <f OtxT1SetRobBitData>
    <f OtxT1GetRobBitData>

*******************************************************************************/
struct OtxT1RobBitDataS {
  OTX_UINT32 chA;  // @field Channel A
  OTX_UINT32 chB;  // @field Channel B
  OTX_UINT32 chC;  // @field Channel C (Only used in ESF format)
  OTX_UINT32 chD;  // @field Channel D (Only used in ESF format)
};

typedef struct OtxT1RobBitDataS OtxT1RobBitDataS;



/*----------------------------------------------------------------------------*\
  E1 Specfic Configuration Types
\*----------------------------------------------------------------------------*/

/*******************************************************************************

  @enum
     E1 Line Codes
   
  @xref
     <t OtxE1ConfigOptionsS>
     <f OtxE1SetConfiguration>

*******************************************************************************/
enum OtxE1LineCodeE {
  OTX_E1_LC_AMI  = 0,   // @emem Alternate Mark Inversion (AMI) Line Coding
  OTX_E1_LC_HDB3 = 1    // @emem High Density Bipolar 3 (HDB3) Line Coding
};

typedef enum OtxE1LineCodeE OtxE1LineCodeE;


/*******************************************************************************

  @enum
     E1 Frame Formats

  @xref
     <t OtxE1ConfigOptionsS>
     <f OtxE1SetConfiguration>

*******************************************************************************/
enum OtxE1FrameFormatE {
  OTX_E1_FF_DOUBLE_FRAME         = 0,   // @emem Double Frame
  OTX_E1_FF_CRC4_MULTIFRAME      = 1,   // @emem CRC4 Multiframe
  OTX_E1_FF_CRC4_MULTIFRAME_G706 = 2,   // @emem CRC4 Multiframe format with modified CRC4 Multiframe alignment
                                        // algorithm (Interworking according to CCITT G.706 Annex B)
  OTX_E1_FF_UNFRAMED             = 3    // @emem Unframed (Transparent) mode. In this mode data for timeslot 0 (normally the frame synchronization timeslot) has to be provide via timeslot 0 of the highway to this device.
};

typedef enum OtxE1FrameFormatE OtxE1FrameFormatE;


/*******************************************************************************

  @enum
     Set the algorithm that is used to detect the Alarm Indication Signal (AIS).
   
  @comm
     According to ETX300233 an AIS alarm is detected if the incoming data stream
     contains less than 3 zeros within 512 bits and a Loss of Frame Alignment is
     indicated. The alarm will be cleared if 3 or more zeros will be detected within
     512 bits or a Frame Alignment Signal (FAS) word is found. According to
     ITU-T G.775 an AIS alarm is detected if the incoming data stream contains less
     than 3 zeros within a doubleframe (512 bits) over a two doubleframe period (1024 bits).
     The alarm will be cleared if 3 or more zeros are detected within a doubleframe
     for two consecutive doubleframes.

  @xref
     <t OtxE1ConfigOptionsS>
     <f OtxE1SetConfiguration>

*******************************************************************************/
enum OtxE1AisDetectModeE {
  OTX_E1_AIS_ETS300233 = 0,   // @emem AIS alarm will be detected according to ETS300233
  OTX_E1_AIS_G775      = 1    // @emem AIS alarm will be detected according to CCITT G.775
};

typedef enum OtxE1AisDetectModeE OtxE1AisDetectModeE;


/*******************************************************************************

  @enum
    Pseudo Random Bit Sequence (PRBS) action type 

  @xref
    <f OtxT1E1SetPrbsConfig>
    <t OtxPrbsPatternE>

*******************************************************************************/
enum OtxPrbsModeE {
  OTX_T1E1_PRBS_MONITOR_ON  = 0,   // @emem Enable PRBS monitor
  OTX_T1E1_PRBS_GENERATE_ON = 1,   // @emem Enable PRBS generate
  OTX_T1E1_PRBS_DISABLE     = 2,   // @emem Disable PRBS monitor or generate
};

typedef enum OtxPrbsModeE OtxPrbsModeE;


/*******************************************************************************

  @enum
    Pseudo Random Bit Sequence (PRBS) Line Loopback Pattern 

  @xref
    <f OtxT1E1SetPrbsConfig>
    <t OtxPrbsModeE>

*******************************************************************************/
enum OtxPrbsPatternE {
  OTX_PRBS_2_15 = 0,   // @emem  Pattern 2^15-1
  OTX_PRBS_2_20 = 1,   // @emem  Pattern 2^20-1
};

typedef enum OtxPrbsPatternE OtxPrbsPatternE;


/*******************************************************************************

  @struct
    E1 Line Interface Configuration Options

  @xref
   <f OtxE1SetConfiguration>
   <f OtxE1GetConfiguration>

*******************************************************************************/
struct OtxE1ConfigOptionsS {
  OtxE1LineCodeE      eTxLineCode;         // @field Line Coding in Transmit Direction
  OtxE1LineCodeE      eRxLineCode;         // @field Line Coding in Receive Direction
  OtxE1FrameFormatE   eTxFrameFormat;      // @field Frame Format in Transmit Direction
  OtxE1FrameFormatE   eRxFrameFormat;      // @field Frame Format in Receive Direction
  OtxE1AisDetectModeE eAisDetection;       // @field Alarm Indication Signal (AIS) Detect mode
  OTX_UINT8           bfSiBits;            // @field E-bit (spare bits for international use) Due to signalling requirements
                                           // the E-bits (SiBits) of frame 13 and frame 15 of the CRC multiframe can be
                                           // used to indicate received errored submultiframes. E-bit of frame 15 is the least
                                           // significant bit (bit 0) of this field. E-bit of frame 13 is bit 1 of this field.
                                           // To set the either E-bit in the in the multiframe
                                           // the corresponding bit position in this field should be turned on.
                                           // E-bit Frame 13, E-bit Frame 15: 00, 01, 10, 11.
                                           // E.g. to set both E-bits (frame 13 and frame 15) set this field to 0x03.
  OTX_UINT8           bfSaBits;            // @field Set the Sa bits (spare bits for national use) transmitted in bit
                                           // positions 4-8 of time-slot 0 of every other (odd) frame. Use bit pattern
                                           // marking the corresponding bit position '1' the bit should be turned on.
                                           // Sa4, Sa5, Sa6, Sa7, Sa8: 00000, 00001, ..., 11111
                                           // <nl> For all Sa bits on nSaBits = 0x1F
  OTX_BOOL            bExtendedHDB3errorDetect;  // @field Selects the error detection mode. When turned off (OTX_FALSE),
                                           // only double violations are detected. When turned on (OTX_TRUE), 0000 strings
                                           // are detected as code violations as well.
  OTX_BOOL            bAutoRegainMultiframe;// @field When enabled (OTX_TRUE), the receiver will search a new basic-
                                           // and multiframing if more than 914 CRC errors have been detected in a
                                           // time interval of one second.
  OTX_BOOL            bAutoResync;         // @field If OTX_TRUE, the Framer will automatically re-synchronize after
                                           // loss of synchronization. If OTX_FALSE the Framer will not synchronize
                                           // automatically but a re-synchronization procedure must be specifically
                                           // initiated by the software.
  OTX_BOOL            bAutoCrcBitInversion; // @field If OTX_TRUE, all CRC bits of one outgoing sub-multiframe are
                                           // inverted in case a CRC error is flagged for the previous received 
                                           // sub-multiframe.
  OTX_BOOL            bAutoRemoteAlarm;    // @field If OTX_TRUE, a Remote Alarm (Yellow Alarm) will be sent automatically
                                           // in the outgoing data stream when the receiver is out of synchronization.
                                           // The alarm will also be automatically removed when the receiver returns to
                                           // the synchronized state.
  OTX_UINT8           nLosSensitivity;     // @field Sets the sensitivity level for the Loss Of Signal (LOS) detection.
                                           // A LOS alarm will be generated if the incoming data stream has no
                                           // transitions during programmable number (N) of consecutive pulse periods.
                                           // The number given as the LOS Sensitivity value will be multiplied by 16 to
                                           // arrive to the pulse count number N. The time for detecting LOS will be
                                           // 16 x LOS Sensitivity x Pulse Length (488ns).
                                           // The maximum time is 16 x 256 x 488ns = 2ms
                                           // <nl> Possible Values: 1-256
                                           // <nl> Example/Default Value:
                                           // LOS Sensitivity  =  128     (16 x 128 x 488ns = 1ms)
  OTX_UINT8           nLosRecovery;        // @field Sets the sensitivity level for the Loss Of Signal (LOS) recovery.
                                           // A LOS alarm will be cleared if in the incoming data a number of Pulses (N)
                                           // occur during the time period set with the LOS Recovery parameter. The time
                                           // interval starts with the first detected pulse transition and a pulse
                                           // count is incremented with every received pulse. If the pulse number is
                                           // greater than the number N set with this parameter, the LOS alarm will be cleared.
                                           // <nl> Possible Values: 1-256
  OTX_UINT16          nLineLength;         // @field Specify the length of the T1 line cable in meters. This information
                                           // is used by the T1 transceivers to adjust the transmit pulse masks according
                                           // to the line length.
                                           // <nl> Possible Values: 0-65535
  OTX_BOOL            bHighTxPower;        // @field When OTX_TRUE the pulses on the line interface will be sent with
                                           // high power. If OTX_FALSE the pulses will be sent with low power.
  OTX_BOOL            bReceiveEqualizer;   // @field The Receive Equalizer can be used match the 18dB ITU-T I.431
                                           // requirement. When the Equalizer is turned off (OTX_FALSE), the Receiver is -10dB (short haul), when
                                           // turned on (OTX_TRUE), the Receiver is -36dB (long haul).
};

typedef struct OtxE1ConfigOptionsS OtxE1ConfigOptionsS;


/*******************************************************************************

  @struct
    Value for the E1 ABCD Bits to be transmitted (used in CAS). See
    the <f OtxE1SetABCDBitData> function.

  @comm
    The A, B, C, and D channels are transmitted in time-slot 16 of a
    E1 multframe.

    The struct holds 30 bits for each channel. The least significant
    bit (bit #0, zero counting) is transmitted first in channel 1,
    frame 1. Bit #15 is then transmitted in channel 16, frame 1.
    Next the bits #1 and #16 are transmitted in channels 2 and 17 in
    frame 2. And Finally, the bits #14 and #29 are transmitted on
    channels 15 and 30 in Frame 15.

    These values will be repeated unless <f OtxE1SetABCDBitData> is called
    again with a different values in this struct.

  @ex Multi-Frame structure for time-slot 16 |

+-----------+-----------+-----------+-----------+     +-----------+
|   TS16    |   TS16    |   TS16    |   TS16    |     |   TS16    |
|  Frame 0  |  Frame 1  |  Frame 2  |  Frame 3  |     |  Frame 15 |
+-----------+-----------+-----------+-----------+ ... +-----------+
| 0000xyxx  | abcd abcd | abcd abcd | abcd abcd |     | abcd abcd |
|           | ch1  ch16 | ch2  ch17 | ch3  ch18 |     | ch15 ch30 |
+-----------+-----------+-----------+-----------+     +-----------+

  @devnote

  When B, C or D are not used, they should have values
         B = 1, C = 0, D = 1.

  It is recommended that the combination of 0000 of A, B, C, and D should
  not be used for signalling purposes for channels 1 to 15.

  @xref
    <f OtxE1SetRobBitData>
    <f OtxE1GetRobBitData>

*******************************************************************************/
struct OtxE1ABCDBitDataS {
  OTX_UINT32 chA;  // @field Channel A
  OTX_UINT32 chB;  // @field Channel B
  OTX_UINT32 chC;  // @field Channel C
  OTX_UINT32 chD;  // @field Channel D
};

typedef struct OtxE1ABCDBitDataS OtxE1ABCDBitDataS;


/*******************************************************************************

  @enum
     E1 time-slot 0 Sa bits (spare bits for national use)

  @xref
    <f OtxE1SetSaBit>
    <f OtxE1GetSaBit>

*******************************************************************************/
enum OtxE1SaBitE {
  OTX_E1_SA4 = 0,       // @emem Sa Bit number 4
  OTX_E1_SA5 = 1,       // @emem Sa Bit number 5
  OTX_E1_SA6 = 2,       // @emem Sa Bit number 6
  OTX_E1_SA7 = 3,       // @emem Sa Bit number 7
  OTX_E1_SA8 = 4        // @emem Sa Bit number 8
};

typedef enum OtxE1SaBitE OtxE1SaBitE;



//***************************************************************************************
#ifdef _TURNSTONE_
//***************************************************************************************


enum  TsT1E1ModeE{
	terminate	= 1,
	monitor		= 2,
	bridge		= 3,
	loopback	= 4,
};
typedef enum TsT1E1ModeE TsT1E1ModeE;

enum TsT1E1LineBuildOutE{
	db_0	= 1,
	db_7_5	= 2,
	db_15	= 3,
	db_22_5	= 4,
};

typedef enum TsT1E1LineBuildOutE TsT1E1LineBuildOutE;



enum TsT1E1ErrorTypeE{
	fas_error = 1,	/* Frame Alignment sequence error */
	mfe_error = 2,	/* Multi Frame error */
	crc_error = 3,  /* crc error insertion */
	cas_error = 4,  /* cas error insertion */
	bit_error = 5,	/* Psuedo random sequence error insertion*/
	bpv_error = 6,  /* Insert bipolar violation error */
};

typedef enum TsT1E1ErrorTypeE TsT1E1ErrorTypeE;



enum TsT1E1PrbsTypeE {
	bits_15 = 0,
	bits_20 = 1
};
typedef enum TsT1E1PrbsTypeE TsT1E1PrbsTypeE;

enum TsT1E1PrbsOutPutE {
	bit_noninverted = 0,
	bit_inverted	= 1	
};
typedef  enum TsT1E1PrbsOutPutE TsT1E1PrbsOutPutE;

struct TsT1E1IoInPrbsConfigOptionsS
{
	TsT1E1PrbsTypeE ePatType;
	TsT1E1PrbsOutPutE ePatOutPut;
	OTX_UINT32 nDuration;      // Duration will be used for Timed Tests
};
typedef struct TsT1E1IoInPrbsConfigOptionsS TsT1E1IoInPrbsConfigOptionsS;

struct TsT1E1IoInStartStopPrbsControlS
{
	OTX_UINT8 bStartPrbs;
};
typedef struct TsT1E1IoInStartStopPrbsControlS TsT1E1IoInStartStopPrbsControlS;

enum ConfigTypeE{
	type_frameformat = 0,
	type_linecode    = 1,
	type_eqswitch    = 2
};
typedef enum ConfigTypeE ConfigTypeE ;

#define TS_REAL_TIME_TIMER_EVENT     L"ts_real_time_seconds_timer"

//***************************************************************************************
#endif  //_TURNSTONE_
//***************************************************************************************


#if defined(__linux__)
#pragma pack()
#elif defined(__BORLANDC__)
#pragma option pop
#else
#pragma pack(pop)
#endif

#ifdef __cplusplus
  } // extern "C"
#endif


/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1998-2003


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/
#endif // OTX_T1E1_DEF_H
